#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/20
#Last Modify: 2018/04/20
# deeper learning insights machine learning page 293

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from sklearn.datasets import load_boston

lb = load_boston()
df = pd.DataFrame(lb.data, columns=lb.feature_names)
df.insert(13, "MEDV", lb.target)
X = df[['RM']].values
y = df[['MEDV']].values

from sklearn.linear_model import RANSACRegressor
from sklearn.linear_model import LinearRegression
# We set the maximum number of iterations of the RANSACRegressor to 100, and
# using min_samples=50 , we set the minimum number of the randomly chosen
# samples to be at least 50. Using the residual_metric parameter, we provided a
# callable lambda function that simply calculates the absolute vertical
# distances between the fitted line and the sample points. By setting the
# residual_threshold parameter to 5.0, we only allowed samples to be included
# in the inlier set if their vertical distance to the fitted line is within 5
# distance units,
ransac = RANSACRegressor(LinearRegression(),
                         max_trials=100,
                         min_samples=50,
                         residual_metric=lambda x: np.sum(np.abs(x), axis=1),
                         residual_threshold=6.0, #NOTE was 5.0
                         random_state=0)
ransac.fit(X, y)

inlier_mask = ransac.inlier_mask_
outlier_mask = np.logical_not(inlier_mask)
line_X = np.arange(3, 10, 1)
line_y_ransac = ransac.predict(line_X[:, np.newaxis])
plt.scatter(X[inlier_mask], y[inlier_mask],
            c='blue', marker='o', label='Inliers')
plt.scatter(X[outlier_mask], y[outlier_mask],
            c='lightgreen', marker='s', label='Outliers')
plt.plot(line_X, line_y_ransac, color='red')
plt.xlabel('Average number of rooms [RM]')
plt.ylabel('Price in $1000\'s [MEDV]')
plt.legend(loc='upper left')
plt.show()

print('Slope: %.3f' % ransac.estimator_.coef_[0])
#Slope: 9.621 #closer results with threshold 6.0
print('Intercept: %.3f' % ransac.estimator_.intercept_)
#Intercept: -37.137 #closer results with threshold 6.0

